package de.desy.language.snl.ui.preferences;

import java.io.File;

import de.desy.language.snl.ui.SNLUiActivator;

public abstract class CompilerOptionsService {

	public static CompilerOptionsService defaultInstance() {
		return new CompilerOptionsService() {
		};
	}

	/**
	 * Returns the compiler path (directory) stored in the preference store if
	 * exists, null otherwise.
	 * 
	 * @param compilerConstant
	 *            The compiler constantc
	 * 
	 * @return The Path as a File-instance or null if no valid path is avail.
	 */
	private File getCompilerPath(PreferenceConstants compilerConstant) {
		String pathToSNC = SNLUiActivator.getDefault().getPreferenceStore()
				.getString(SNLUiActivator.PLUGIN_ID + compilerConstant);

		if (pathToSNC == null || pathToSNC.trim().length() < 1) {
			return null;
		}

		File result = new File(pathToSNC);

		return result;
	}

	/**
	 * Returns the SNL-Compiler path (directory) stored in the preference store
	 * if exists, null otherwise.
	 * 
	 * @return The Path as a File-instance or null if no valid path is avail.
	 */
	public File getSNLCompilerPath() {
		File compilerPath = getCompilerPath(PreferenceConstants.SNC_LOCATION_POST_FIX);
		if (compilerPath != null && !compilerPath.isDirectory()) {
			return null;
		} else {
			return compilerPath;
		}

	}

	/**
	 * Returns the C-Compiler path (directory) stored in the preference store if
	 * exists, null otherwise.
	 * 
	 * @return The Path as a File-instance or null if no valid path is avail.
	 */
	public File getCCompilerPath() {
		return getCompilerPath(PreferenceConstants.C_LOCATION_POST_FIX);
	}
}
